/*
 * cameraTest.c
 *
 *  Created on: 28 Jan 2025
 *      Author: user
 */
/*
#include "cameraTest.h"
#include "servo.h"
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "MK66F18.h"
#include "fsl_debug_console.h"

AT_NONCACHEABLE_SECTION (  uint16_t camera_adc_output[IMAGE_SIZE]);
AT_NONCACHEABLE_SECTION (  uint16_t c_camera_adc_output[IMAGE_SIZE]);\
AT_NONCACHEABLE_SECTION (  uint16_t buffer[IMAGE_SIZE]);

 int16_t prev_deviations[5];
 int16_t camera_data_filtered[IMAGE_SIZE];
 const float filter[];
 volatile uint16_t current_pixel_number = 0;
 volatile int threshold = 3300;
 uint16_t average = 0;
 volatile uint32_t right_filtered_value_max;
 volatile uint32_t left_filtered_value_max;
 uint16_t min_difference = 500;
 bool rightedge = false;
 bool leftedge = false;
 int leftindex = 0;
 int rightindex = 0;
 int middleindex = 64;
 int deviation = 0;

//ADC1_IRQn interrupt handler
void ADC1_IRQHANDLER(void) {
    uint32_t result_values[2] = {0};
    for (int i = 0; i < 2; i++) {
        uint32_t status = ADC16_GetChannelStatusFlags(ADC1_PERIPHERAL, i);
        if (status == kADC16_ChannelConversionDoneFlag) {
            result_values[i] = ADC16_GetChannelConversionValue(ADC1_PERIPHERAL, i);
        }
    }
    // Save the current pixel value to the camera_adc_output array

    camera_adc_output[current_pixel_number] = result_values[0];
    current_pixel_number++;

    if (current_pixel_number == 128){
    	memcpy (c_camera_adc_output, camera_adc_output, sizeof(camera_adc_output));
    	current_pixel_number = 0;
    }


    #if defined _CORTEX_M && (_CORTEX_M == 4U)
        __DSB();
    #endif
}




// FTM1_IRQn interrupt handler
void FTM1_IRQHANDLER(void) {
  uint32_t intStatus;
  static int16_t parallax1_si_count=0;

  // Reading all interrupt flags of status register
  intStatus = FTM_GetStatusFlags(FTM1_PERIPHERAL);
  FTM_ClearStatusFlags(FTM1_PERIPHERAL, intStatus);

  //We only have a single interrupt source for the FTM1 so we do not need to select (yet)
  // This is the counter for the SI cycles, it is counted once per clock cycle, only in the first clock cycle the SI signal is set to high
  parallax1_si_count++;

  // After the first cycle, the SI Signal for the Parallax Camera is cleared
  if(parallax1_si_count==1){
      GPIO_PortClear(BOARD_PARALLAX1_S1_PTA9_GPIO, BOARD_PARALLAX1_S1_PTA9_GPIO_PIN_MASK);
      // The write pointer into the array is set to zero
      current_pixel_number=0;
  }
  // We start the conversion cycle of the ADC if the si signal is in the 1st to 128th cycle
  // Please note that we start triggering the conversion directly after the SI signal has been cleared
  if((parallax1_si_count >= 1)&&(parallax1_si_count <= 128)){
        ADC16_SetChannelConfig(ADC1_PERIPHERAL, ADC1_CH0_CONTROL_GROUP, &ADC1_channelsConfig[0]);
  }
  // We reset the SI cycle counter after a certain time period in order to start again another capture of 128 pixels
  // The time after the 128th cycle and the restart has to be 20us according to the specification,
  // in practice it seems that we can also wait longer and have a longer integration time
  if(parallax1_si_count>SI_CYCLE_LIMIT){
      parallax1_si_count=0;
      // As we are resetting the SI count, we also set the SI pin to high
      GPIO_PortSet(BOARD_PARALLAX1_S1_PTA9_GPIO, BOARD_PARALLAX1_S1_PTA9_GPIO_PIN_MASK);
  }

  // Add for ARM errata 838869, affects Cortex-M4, Cortex-M4F
     Store immediate overlapping exception return operation might vector to incorrect interrupt.
  #if defined _CORTEX_M && (_CORTEX_M == 4U)
    __DSB();
  #endif
}



void datawrite(){

	uint16_t datacamera[IMAGE_SIZE];
	FILE* cameraFile = NULL;
	cameraFile = fopen("datacamera.bin","w+b");

	if (cameraFile != NULL){
		fwrite(buffer, sizeof(buffer[0]), IMAGE_SIZE, cameraFile);
		fseek(cameraFile, 0, SEEK_SET);
		fread(datacamera, sizeof(buffer[0]), IMAGE_SIZE, cameraFile);
	}else{
		printf("No file found");
	}

	for (int i=0; i<IMAGE_SIZE; i++){
		printf("Result %d : %d\n" , i , datacamera[i]);
	}


}



void calibrate_camera(){

	memcpy (buffer, c_camera_adc_output, sizeof(camera_adc_output));
	for(int i=0; i<IMAGE_SIZE; i++){
		if(buffer[i] > threshold){
			buffer[i]=threshold;
		}
	}
}



void convolution(){

	for(int i=0; i<123; i++){
		for(int j=0; j<6; j++){
			average += (buffer[j+i] * filter[j]);
		}
		camera_data_filtered[i] = abs(average);
		average = 0;
	}
}



void edge_detection() {
    right_filtered_value_max = 0;
    left_filtered_value_max = 0;

    for (int i = 0; i < middleindex; i++) {
        if (right_filtered_value_max < buffer[i]) {
            right_filtered_value_max = buffer[i];
            rightindex = i;
        }
    }

    if (right_filtered_value_max < threshold) {
        rightedge = true;
    } else {
        rightedge = false;
    }

    for (int i = middleindex; i < IMAGE_SIZE; i++) {
        if (left_filtered_value_max < buffer[i]) {
            left_filtered_value_max = buffer[i];
            leftindex = i;
        }
    }

    if (left_filtered_value_max < threshold) {
        leftedge = true;
    } else {
        leftedge = false;
    }

    if (rightedge && leftedge) {
        int dec_middle = (rightindex + leftindex) / 2;
        deviation = dec_middle - middleindex;
    } else if (leftedge) {
        deviation = leftindex - middleindex;
    } else if (rightedge) {
        deviation = rightindex - middleindex;
    } else {
        deviation = 0;
    }
}

void move(){
	 if (prev_deviations[5] > 5){
		 turnLeft();
	 }else if (prev_deviations[5] < -5){
		 turnRight();
	 }else{
		 centerServo();
	 }

 }

void deviationupdate() {
	prev_deviations[5] = prev_deviations[4];
	prev_deviations[4] = prev_deviations[3];
	prev_deviations[3] = prev_deviations[2];
	prev_deviations[2] = prev_deviations[1];
	prev_deviations[1] = prev_deviations[0];
	prev_deviations[0] = deviation;
}

int findTrackCenter(int *leftEdge, int *rightEdge) {
    int foundLeft = 0, foundRight = 0;

    // Debugging: Print ADC values
    printf("ADC Values: ");
    for (int i = 0; i < IMAGE_SIZE; i += 10) {
        printf("%d ", camera_adc_output[i]);
    }
    printf("\n");

    // Find left black edge (moving from left to right)
    for (int i = 10; i < IMAGE_SIZE / 2; i++) {  // Start from index 10 to avoid noise
        if (camera_adc_output[i] < THRESHOLD) {  // Black detected
            *leftEdge = i;
            foundLeft = 1;
            break;
        }
    }

    // Find right black edge (moving from right to left)
    for (int i = IMAGE_SIZE - 10; i > IMAGE_SIZE / 2; i--) {  // Start from index 118 to avoid noise
        if (camera_adc_output[i] < THRESHOLD) {  // Black detected
            *rightEdge = i;
            foundRight = 1;
            break;
        }
    }

    // If no edges found, assume track is wide open
    if (!foundLeft) {*leftEdge = 20; }// Assume some left margin
    if (!foundRight) {*rightEdge = IMAGE_SIZE - 20;} // Assume some right margin

    // Debugging: Show detected edges
    printf("Detected Left Edge: %d, Right Edge: %d\n", *leftEdge, *rightEdge);

    return 0;  // Updating pointers directly
}

void updateSteering(int leftEdge, int rightEdge) {
    int trackMiddle = (leftEdge + rightEdge) / 2;
    int deviation = trackMiddle - TRACK_CENTER;

    printf("Track Middle: %d, Deviation: %d\n", trackMiddle, deviation);

    // If trackMiddle is too far left, turn right
    if (trackMiddle < TRACK_CENTER - 10) {
        printf("Drifting LEFT! Turning RIGHT ⬅️\n");
        turnRight();
    }
    // If trackMiddle is too far right, turn left
    else if (trackMiddle > TRACK_CENTER + 10) {
        printf("Drifting RIGHT! Turning LEFT ➡️\n");
        turnLeft();
    }
    // Otherwise, go straight
    else {
        printf("Keeping straight.\n");
        centerServo();
    }
}*/
