/*
 * led.c
 *
 *  Created on: 12 Jan 2025
 *      Author: user
 */
#include "led.h"
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "MK66F18.h"
#include "fsl_debug_console.h"

// Buffer for storing LED data
AT_NONCACHEABLE_SECTION(uint8_t srcAddr[BUFF_LENGTH]);

// Public variables
volatile bool g_Transfer_Done = false;
volatile bool isTransferCompleted = false;

// Callback for SPI transfer completion
void ws2812b_spi0_edma_handler(SPI_Type *base, dspi_master_edma_handle_t *handle, status_t status, void *userData) {
    if (status == kStatus_Success) {
        PRINTF("This is DSPI master EDMA callback.\r\n");
    }
    isTransferCompleted = true;
}

// Function to send LED data via SPI
void sendLEDData(uint8_t *srcAddr) {
	edma_config_t userConfig;
	edma_transfer_config_t transferConfig;
    dspi_transfer_t masterXfer;

    isTransferCompleted = false;
    masterXfer.txData = srcAddr;
    masterXfer.rxData = NULL;
    masterXfer.dataSize = BUFF_LENGTH;
    masterXfer.configFlags = kDSPI_MasterCtar0 | kDSPI_MasterPcs0 | kDSPI_MasterPcsContinuous;

    if (kStatus_Success != DSPI_MasterTransferEDMA(SPI0_PERIPHERAL, &ws2812b_spi0_edma_handle, &masterXfer)) {
        PRINTF("Error: Failed to start SPI transfer.\r\n");
    }

    while (!isTransferCompleted) {
    }
}

// Function to set all LEDs to white
void whiteLight(void){
	int i,j;
	for (i = 0; i < BUFF_LENGTH; i += 3) {
		// 0b110110100100100100100100 = 0xDA4924
		srcAddr[i + 0] = 0xD3;
		srcAddr[i + 1] = 0x49;
		srcAddr[i + 2] = 0x24;
	}
    sendLEDData(srcAddr);
}

void rightLight(void){
	int i,j;
	for (i = 0; i < BUFF_LENGTH/2; i += 3 * 3) {
		// 0b110110100100100100100100 = 0xDA4924
		srcAddr[i + 0] = 0xD3;
		srcAddr[i + 1] = 0x49;
		srcAddr[i + 2] = 0x24;
	}
    sendLEDData(srcAddr);
}

void leftLight(void){
	int i,j;
	for (i = BUFF_LENGTH/2; i < BUFF_LENGTH; i += 3 * 3) {
		// 0b110110100100100100100100 = 0xDA4924
		srcAddr[i + 0] = 0xD3;
		srcAddr[i + 1] = 0x49;
		srcAddr[i + 2] = 0x24;
	}
    sendLEDData(srcAddr);
}

void turnOff(void){
	for (int i = 0; i < BUFF_LENGTH; i += 3) {
		// 0b100100100100100100100100 = 0x924924
		srcAddr[i + 0] = 0x92;
		srcAddr[i + 1] = 0x49;
		srcAddr[i + 2] = 0x24;
	}
	sendLEDData(srcAddr);
}
